# Generated by Django 4.1.2 on 2022-12-02 22:41

import _socket
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_crf.model_mixins.crf_status_model_mixin
import edc_model.models.fields.duration
import edc_model.validators.date
import edc_model_fields.fields.other_charfield
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("intecomm_lists", "0003_dmmanagement_dmmanagement_intecomm_li_id_b4d41f_idx"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("intecomm_subject", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicaldminitialreview",
            name="managed_by",
        ),
        migrations.RemoveField(
            model_name="historicalotherbaselinedata",
            name="num_vegetables_eaten",
        ),
        migrations.RemoveField(
            model_name="historicalotherbaselinedata",
            name="use_salt_on_food",
        ),
        migrations.RemoveField(
            model_name="historicalotherbaselinedata",
            name="work_activity",
        ),
        migrations.RemoveField(
            model_name="historicalotherbaselinedata",
            name="work_activity_days",
        ),
        migrations.RemoveField(
            model_name="historicalotherbaselinedata",
            name="work_activity_exercise",
        ),
        migrations.RemoveField(
            model_name="historicalotherbaselinedata",
            name="work_activity_exercise_time",
        ),
        migrations.RemoveField(
            model_name="otherbaselinedata",
            name="num_vegetables_eaten",
        ),
        migrations.RemoveField(
            model_name="otherbaselinedata",
            name="use_salt_on_food",
        ),
        migrations.RemoveField(
            model_name="otherbaselinedata",
            name="work_activity",
        ),
        migrations.RemoveField(
            model_name="otherbaselinedata",
            name="work_activity_days",
        ),
        migrations.RemoveField(
            model_name="otherbaselinedata",
            name="work_activity_exercise",
        ),
        migrations.RemoveField(
            model_name="otherbaselinedata",
            name="work_activity_exercise_time",
        ),
        migrations.AddField(
            model_name="dminitialreview",
            name="managed_by_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="dminitialreview",
            name="singleton_field",
            field=models.CharField(
                help_text="auto updated for unique constraint",
                max_length=50,
                null=True,
                unique=True,
                verbose_name="subject identifier",
            ),
        ),
        migrations.AddField(
            model_name="historicaldminitialreview",
            name="managed_by_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicaldminitialreview",
            name="singleton_field",
            field=models.CharField(
                db_index=True,
                help_text="auto updated for unique constraint",
                max_length=50,
                null=True,
                verbose_name="subject identifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivinitialreview",
            name="singleton_field",
            field=models.CharField(
                db_index=True,
                help_text="auto updated for unique constraint",
                max_length=50,
                null=True,
                verbose_name="subject identifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalhtninitialreview",
            name="singleton_field",
            field=models.CharField(
                db_index=True,
                help_text="auto updated for unique constraint",
                max_length=50,
                null=True,
                verbose_name="subject identifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalotherbaselinedata",
            name="activity_combined_mn_avg_day",
            field=models.IntegerField(
                blank=True,
                help_text="Number of minutes in a typical day",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(1080),
                ],
                verbose_name="On an average day, how many MINUTES do you spend on all physical activities combined (moderate or vigorous-intensity work activities, exercise, sports and fitness)",
            ),
        ),
        migrations.AddField(
            model_name="historicalotherbaselinedata",
            name="activity_exercise_days_per_wk",
            field=models.IntegerField(
                blank=True,
                help_text="Number of days in a typical week",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(
                        7, message="Maximum days in a week is 7"
                    ),
                ],
                verbose_name="How many DAYS do you do personal exercise, sports or fitness activities? (in a typical week)",
            ),
        ),
        migrations.AddField(
            model_name="historicalotherbaselinedata",
            name="activity_work",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="See interviewer notes above.",
                max_length=15,
                null=True,
                verbose_name="Does your work involve moderate or vigorous-intensity physical activity",
            ),
        ),
        migrations.AddField(
            model_name="historicalotherbaselinedata",
            name="activity_work_days_per_wk",
            field=models.IntegerField(
                blank=True,
                help_text="Number of days in a typical week",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(7),
                ],
                verbose_name="If yes, how many DAYS do you do these physical activities as part of your work? (in a typical week)",
            ),
        ),
        migrations.AddField(
            model_name="historicalotherbaselinedata",
            name="adds_salt_to_food",
            field=models.CharField(
                choices=[
                    ("always", "Always"),
                    ("very_often", "Very Often"),
                    ("sometimes", "Sometimes"),
                    ("rarely", "Rarely"),
                    ("NEVER", "Never"),
                ],
                max_length=15,
                null=True,
                verbose_name="Do you add salt to your food in addition to what has been added when cooking?",
            ),
        ),
        migrations.AddField(
            model_name="historicalotherbaselinedata",
            name="num_fruits_eaten",
            field=models.IntegerField(
                default=0,
                help_text="Number per day",
                verbose_name="On average, how many vegetables do you eat in a day?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalotherbaselinedata",
            name="num_leafy_vegs_eaten",
            field=models.IntegerField(
                default=0,
                help_text="Number per day",
                verbose_name="On average, how many green leafy vegetables do you eat in a day?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalotherbaselinedata",
            name="num_other_vegs_eaten",
            field=models.IntegerField(
                default=0,
                help_text="Other vegetables like carrots, tomatoes, bell papers etc. Number per day",
                verbose_name="On average, how many other vegetables do you eat in a day?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalotherbaselinedata",
            name="smoker_current_duration",
            field=edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalotherbaselinedata",
            name="smoker_current_duration_estimated_date",
            field=edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="hivinitialreview",
            name="singleton_field",
            field=models.CharField(
                help_text="auto updated for unique constraint",
                max_length=50,
                null=True,
                unique=True,
                verbose_name="subject identifier",
            ),
        ),
        migrations.AddField(
            model_name="htninitialreview",
            name="singleton_field",
            field=models.CharField(
                help_text="auto updated for unique constraint",
                max_length=50,
                null=True,
                unique=True,
                verbose_name="subject identifier",
            ),
        ),
        migrations.AddField(
            model_name="otherbaselinedata",
            name="activity_combined_mn_avg_day",
            field=models.IntegerField(
                blank=True,
                help_text="Number of minutes in a typical day",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(1080),
                ],
                verbose_name="On an average day, how many MINUTES do you spend on all physical activities combined (moderate or vigorous-intensity work activities, exercise, sports and fitness)",
            ),
        ),
        migrations.AddField(
            model_name="otherbaselinedata",
            name="activity_exercise_days_per_wk",
            field=models.IntegerField(
                blank=True,
                help_text="Number of days in a typical week",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(
                        7, message="Maximum days in a week is 7"
                    ),
                ],
                verbose_name="How many DAYS do you do personal exercise, sports or fitness activities? (in a typical week)",
            ),
        ),
        migrations.AddField(
            model_name="otherbaselinedata",
            name="activity_work",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="See interviewer notes above.",
                max_length=15,
                null=True,
                verbose_name="Does your work involve moderate or vigorous-intensity physical activity",
            ),
        ),
        migrations.AddField(
            model_name="otherbaselinedata",
            name="activity_work_days_per_wk",
            field=models.IntegerField(
                blank=True,
                help_text="Number of days in a typical week",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(7),
                ],
                verbose_name="If yes, how many DAYS do you do these physical activities as part of your work? (in a typical week)",
            ),
        ),
        migrations.AddField(
            model_name="otherbaselinedata",
            name="adds_salt_to_food",
            field=models.CharField(
                choices=[
                    ("always", "Always"),
                    ("very_often", "Very Often"),
                    ("sometimes", "Sometimes"),
                    ("rarely", "Rarely"),
                    ("NEVER", "Never"),
                ],
                max_length=15,
                null=True,
                verbose_name="Do you add salt to your food in addition to what has been added when cooking?",
            ),
        ),
        migrations.AddField(
            model_name="otherbaselinedata",
            name="num_fruits_eaten",
            field=models.IntegerField(
                default=0,
                help_text="Number per day",
                verbose_name="On average, how many vegetables do you eat in a day?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="otherbaselinedata",
            name="num_leafy_vegs_eaten",
            field=models.IntegerField(
                default=0,
                help_text="Number per day",
                verbose_name="On average, how many green leafy vegetables do you eat in a day?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="otherbaselinedata",
            name="num_other_vegs_eaten",
            field=models.IntegerField(
                default=0,
                help_text="Other vegetables like carrots, tomatoes, bell papers etc. Number per day",
                verbose_name="On average, how many other vegetables do you eat in a day?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="otherbaselinedata",
            name="smoker_current_duration",
            field=edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="otherbaselinedata",
            name="smoker_current_duration_estimated_date",
            field=edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
        ),
        migrations.RemoveField(
            model_name="dminitialreview",
            name="managed_by",
        ),
        migrations.AlterField(
            model_name="historicalotherbaselinedata",
            name="education",
            field=models.CharField(
                choices=[
                    ("no_formal_education", "No Formal Education"),
                    ("primary", "Up to primary"),
                    ("secondary", "Up to secondary / high school"),
                    ("post_secondary", "College or similar higher institution"),
                    ("tertiary", "university educated"),
                ],
                max_length=25,
                verbose_name="How much formal education does the patient have?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalotherbaselinedata",
            name="employment_status",
            field=models.CharField(
                choices=[
                    ("professional", "Professional / office work / business"),
                    ("manual_work", "Skilled / Unskilled manual work"),
                    ("housewife", "Housewife"),
                    ("unemployed", "Not working / seeking work"),
                    ("unemployed_not_seeking", "Not working / NOT seeking work"),
                    ("retired", "Retired"),
                    ("OTHER", "Other, please specify"),
                ],
                max_length=25,
                verbose_name="What is the patient's employment status?",
            ),
        ),
        migrations.AlterField(
            model_name="otherbaselinedata",
            name="education",
            field=models.CharField(
                choices=[
                    ("no_formal_education", "No Formal Education"),
                    ("primary", "Up to primary"),
                    ("secondary", "Up to secondary / high school"),
                    ("post_secondary", "College or similar higher institution"),
                    ("tertiary", "university educated"),
                ],
                max_length=25,
                verbose_name="How much formal education does the patient have?",
            ),
        ),
        migrations.AlterField(
            model_name="otherbaselinedata",
            name="employment_status",
            field=models.CharField(
                choices=[
                    ("professional", "Professional / office work / business"),
                    ("manual_work", "Skilled / Unskilled manual work"),
                    ("housewife", "Housewife"),
                    ("unemployed", "Not working / seeking work"),
                    ("unemployed_not_seeking", "Not working / NOT seeking work"),
                    ("retired", "Retired"),
                    ("OTHER", "Other, please specify"),
                ],
                max_length=25,
                verbose_name="What is the patient's employment status?",
            ),
        ),
        migrations.CreateModel(
            name="SocialHarms",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                ("consent_model", models.CharField(editable=False, max_length=50, null=True)),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        help_text="for example, why some data is still pending",
                        null=True,
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "partner",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="Does your PARTNER know about your participation in the study?",
                    ),
                ),
                (
                    "partner_disclosure",
                    models.CharField(
                        choices=[
                            ("voluntary", "Voluntary disclosure"),
                            ("involuntary", "Involuntary disclosure"),
                            ("by_others", "Others told them without permission"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="(e.g., they saw study material, or I had to disclose DM/HTN/HIV",
                        max_length=25,
                        verbose_name="If Yes, how did they find out?",
                    ),
                ),
                (
                    "family",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="Does your FAMILY know about your participation in the study?",
                    ),
                ),
                (
                    "family_disclosure",
                    models.CharField(
                        choices=[
                            ("voluntary", "Voluntary disclosure"),
                            ("involuntary", "Involuntary disclosure"),
                            ("by_others", "Others told them without permission"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="(e.g., they saw study material, or I had to disclose DM/HTN/HIV",
                        max_length=25,
                        verbose_name="If Yes, how did they find out?",
                    ),
                ),
                (
                    "friends",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Do any of your FRIENDS know about your participation in the study?",
                    ),
                ),
                (
                    "friends_disclosure",
                    models.CharField(
                        choices=[
                            ("voluntary", "Voluntary disclosure"),
                            ("involuntary", "Involuntary disclosure"),
                            ("by_others", "Others told them without permission"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="(e.g., they saw study material, or I had to disclose DM/HTN/HIV",
                        max_length=25,
                        verbose_name="If Yes, how did they find out?",
                    ),
                ),
                (
                    "coworkers",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="Do any of your CO-WORKERS know about your participation in the study?",
                    ),
                ),
                (
                    "coworkers_disclosure",
                    models.CharField(
                        choices=[
                            ("voluntary", "Voluntary disclosure"),
                            ("involuntary", "Involuntary disclosure"),
                            ("by_others", "Others told them without permission"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="(e.g., they saw study material, or I had to disclose DM/HTN/HIV",
                        max_length=25,
                        verbose_name="If Yes, how did they find out?",
                    ),
                ),
                (
                    "partner_impact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="... with your partner?",
                    ),
                ),
                (
                    "partner_impact_severity",
                    models.CharField(
                        choices=[
                            ("minor", "Minor"),
                            ("moderate", "Moderate"),
                            ("major", "Major"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Severity of impact",
                    ),
                ),
                (
                    "partner_impact_status",
                    models.CharField(
                        choices=[
                            ("resolved", "Resolved"),
                            ("ongoing", "Ongoing"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Status of impact",
                    ),
                ),
                (
                    "partner_impact_help",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Would you like help",
                    ),
                ),
                (
                    "partner_impact_referal",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Has the participant been referred for further help",
                    ),
                ),
                (
                    "family_impact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="... with your family?",
                    ),
                ),
                (
                    "family_impact_severity",
                    models.CharField(
                        choices=[
                            ("minor", "Minor"),
                            ("moderate", "Moderate"),
                            ("major", "Major"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Severity of impact",
                    ),
                ),
                (
                    "family_impact_status",
                    models.CharField(
                        choices=[
                            ("resolved", "Resolved"),
                            ("ongoing", "Ongoing"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Status of impact",
                    ),
                ),
                (
                    "family_impact_help",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Would you like help",
                    ),
                ),
                (
                    "family_impact_referal",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Has the participant been referred for further help",
                    ),
                ),
                (
                    "friends_impact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="... with your friends?",
                    ),
                ),
                (
                    "friends_impact_severity",
                    models.CharField(
                        choices=[
                            ("minor", "Minor"),
                            ("moderate", "Moderate"),
                            ("major", "Major"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Severity of impact",
                    ),
                ),
                (
                    "friends_impact_status",
                    models.CharField(
                        choices=[
                            ("resolved", "Resolved"),
                            ("ongoing", "Ongoing"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Status of impact",
                    ),
                ),
                (
                    "friends_impact_help",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Would you like help",
                    ),
                ),
                (
                    "friends_impact_referal",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Has the participant been referred for further help",
                    ),
                ),
                (
                    "coworkers_impact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="... with your co-workers?",
                    ),
                ),
                (
                    "coworkers_impact_severity",
                    models.CharField(
                        choices=[
                            ("minor", "Minor"),
                            ("moderate", "Moderate"),
                            ("major", "Major"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Severity of impact",
                    ),
                ),
                (
                    "coworkers_impact_status",
                    models.CharField(
                        choices=[
                            ("resolved", "Resolved"),
                            ("ongoing", "Ongoing"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Status of impact",
                    ),
                ),
                (
                    "coworkers_impact_help",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Would you like help",
                    ),
                ),
                (
                    "coworkers_impact_referal",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Has the participant been referred for further help",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="intecomm_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Social Harms",
                "verbose_name_plural": "Social Harms",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
            },
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalSocialHarms",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                ("consent_model", models.CharField(editable=False, max_length=50, null=True)),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, primary_key=True, serialize=False
                    ),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        help_text="for example, why some data is still pending",
                        null=True,
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "partner",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="Does your PARTNER know about your participation in the study?",
                    ),
                ),
                (
                    "partner_disclosure",
                    models.CharField(
                        choices=[
                            ("voluntary", "Voluntary disclosure"),
                            ("involuntary", "Involuntary disclosure"),
                            ("by_others", "Others told them without permission"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="(e.g., they saw study material, or I had to disclose DM/HTN/HIV",
                        max_length=25,
                        verbose_name="If Yes, how did they find out?",
                    ),
                ),
                (
                    "family",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="Does your FAMILY know about your participation in the study?",
                    ),
                ),
                (
                    "family_disclosure",
                    models.CharField(
                        choices=[
                            ("voluntary", "Voluntary disclosure"),
                            ("involuntary", "Involuntary disclosure"),
                            ("by_others", "Others told them without permission"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="(e.g., they saw study material, or I had to disclose DM/HTN/HIV",
                        max_length=25,
                        verbose_name="If Yes, how did they find out?",
                    ),
                ),
                (
                    "friends",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Do any of your FRIENDS know about your participation in the study?",
                    ),
                ),
                (
                    "friends_disclosure",
                    models.CharField(
                        choices=[
                            ("voluntary", "Voluntary disclosure"),
                            ("involuntary", "Involuntary disclosure"),
                            ("by_others", "Others told them without permission"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="(e.g., they saw study material, or I had to disclose DM/HTN/HIV",
                        max_length=25,
                        verbose_name="If Yes, how did they find out?",
                    ),
                ),
                (
                    "coworkers",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="Do any of your CO-WORKERS know about your participation in the study?",
                    ),
                ),
                (
                    "coworkers_disclosure",
                    models.CharField(
                        choices=[
                            ("voluntary", "Voluntary disclosure"),
                            ("involuntary", "Involuntary disclosure"),
                            ("by_others", "Others told them without permission"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="(e.g., they saw study material, or I had to disclose DM/HTN/HIV",
                        max_length=25,
                        verbose_name="If Yes, how did they find out?",
                    ),
                ),
                (
                    "partner_impact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="... with your partner?",
                    ),
                ),
                (
                    "partner_impact_severity",
                    models.CharField(
                        choices=[
                            ("minor", "Minor"),
                            ("moderate", "Moderate"),
                            ("major", "Major"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Severity of impact",
                    ),
                ),
                (
                    "partner_impact_status",
                    models.CharField(
                        choices=[
                            ("resolved", "Resolved"),
                            ("ongoing", "Ongoing"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Status of impact",
                    ),
                ),
                (
                    "partner_impact_help",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Would you like help",
                    ),
                ),
                (
                    "partner_impact_referal",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Has the participant been referred for further help",
                    ),
                ),
                (
                    "family_impact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="... with your family?",
                    ),
                ),
                (
                    "family_impact_severity",
                    models.CharField(
                        choices=[
                            ("minor", "Minor"),
                            ("moderate", "Moderate"),
                            ("major", "Major"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Severity of impact",
                    ),
                ),
                (
                    "family_impact_status",
                    models.CharField(
                        choices=[
                            ("resolved", "Resolved"),
                            ("ongoing", "Ongoing"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Status of impact",
                    ),
                ),
                (
                    "family_impact_help",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Would you like help",
                    ),
                ),
                (
                    "family_impact_referal",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Has the participant been referred for further help",
                    ),
                ),
                (
                    "friends_impact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="... with your friends?",
                    ),
                ),
                (
                    "friends_impact_severity",
                    models.CharField(
                        choices=[
                            ("minor", "Minor"),
                            ("moderate", "Moderate"),
                            ("major", "Major"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Severity of impact",
                    ),
                ),
                (
                    "friends_impact_status",
                    models.CharField(
                        choices=[
                            ("resolved", "Resolved"),
                            ("ongoing", "Ongoing"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Status of impact",
                    ),
                ),
                (
                    "friends_impact_help",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Would you like help",
                    ),
                ),
                (
                    "friends_impact_referal",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Has the participant been referred for further help",
                    ),
                ),
                (
                    "coworkers_impact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        max_length=25,
                        verbose_name="... with your co-workers?",
                    ),
                ),
                (
                    "coworkers_impact_severity",
                    models.CharField(
                        choices=[
                            ("minor", "Minor"),
                            ("moderate", "Moderate"),
                            ("major", "Major"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Severity of impact",
                    ),
                ),
                (
                    "coworkers_impact_status",
                    models.CharField(
                        choices=[
                            ("resolved", "Resolved"),
                            ("ongoing", "Ongoing"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Status of impact",
                    ),
                ),
                (
                    "coworkers_impact_help",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Would you like help",
                    ),
                ),
                (
                    "coworkers_impact_referal",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=25,
                        verbose_name="Has the participant been referred for further help",
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="intecomm_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Social Harms",
                "verbose_name_plural": "historical Social Harms",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.AddField(
            model_name="dminitialreview",
            name="managed_by",
            field=models.ManyToManyField(
                to="intecomm_lists.dmmanagement",
                verbose_name="How is the patient's diabetes managed?",
            ),
        ),
        migrations.AddIndex(
            model_name="socialharms",
            index=models.Index(
                fields=["subject_visit", "site", "id"], name="intecomm_su_subject_99ad70_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="socialharms",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="intecomm_su_subject_7a815b_idx",
            ),
        ),
    ]



from django.db import models
from edc_constants.choices import YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model.models import BaseUuidModel
from edc_model_fields.fields import OtherCharField

from intecomm_lists.models import ClinicServices, HealthServices

from ..model_mixins import CrfModelMixin


class ReasonForVisit(CrfModelMixin, BaseUuidModel):
    health_services = models.ManyToManyField(
        HealthServices,
        related_name="health_services",
        verbose_name="Which health service(s) is the patient here for today?",
    )

    clinic_services = models.ManyToManyField(
        ClinicServices,
        related_name="clinic_services",
        verbose_name="Why is the patient at the clinic?",
    )

    clinic_services_other = OtherCharField()

    refill_hiv = models.CharField(
        verbose_name="Is the patient refilling HIV medications?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    refill_dm = models.CharField(
        verbose_name="Is the patient refilling Diabetes medications?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    refill_htn = models.CharField(
        verbose_name="Is the patient refilling Hypertension medications?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Reason for Visit"
        verbose_name_plural = "Reason for Visits"



from django.db import models
from edc_constants.choices import YES_NO
from edc_constants.constants import NO
from edc_model import duration_to_date
from edc_model.models import BaseUuidModel, DurationYMDField

from ..model_mixins import CrfModelMixin


class ComplicationsBaseline(CrfModelMixin, BaseUuidModel):
    # stroke
    stroke = models.CharField(
        verbose_name="Stroke",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    stroke_ago = DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    stroke_estimated_date = models.DateField(
        verbose_name="Estimated date of stroke",
        null=True,
        blank=True,
        editable=False,
    )

    # heart_attack
    heart_attack = models.CharField(
        verbose_name="Heart attack / heart failure",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    heart_attack_ago = DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    heart_attack_estimated_date = models.DateField(
        verbose_name="Estimated date of heart attack / heart failure",
        null=True,
        blank=True,
        editable=False,
    )

    # renal
    renal_disease = models.CharField(
        verbose_name="Renal (kidney) disease",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    renal_disease_ago = DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    renal_disease_estimated_date = models.DateField(
        verbose_name="Estimated date of renal_disease",
        null=True,
        blank=True,
        editable=False,
    )
    # vision
    vision = models.CharField(
        verbose_name="Vision problems (e.g. blurred vision)",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    vision_ago = DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    vision_estimated_date = models.DateField(
        verbose_name="Estimated date of vision problems",
        null=True,
        blank=True,
        editable=False,
    )

    # numbness
    numbness = models.CharField(
        verbose_name="Numbness / burning sensation",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    numbness_ago = DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    numbness_estimated_date = models.DateField(
        verbose_name="Estimated date of numbness",
        null=True,
        blank=True,
        editable=False,
    )

    # foot ulcers
    foot_ulcers = models.CharField(
        verbose_name="Foot ulcers",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    foot_ulcers_ago = DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    foot_ulcers_estimated_date = models.DateField(
        verbose_name="Estimated date of foot ulcers",
        null=True,
        blank=True,
        editable=False,
    )

    complications = models.CharField(
        verbose_name="Are there any other major complications to report?",
        max_length=25,
        choices=YES_NO,
        default=NO,
    )

    complications_other = models.TextField(
        null=True,
        blank=True,
        help_text="Please include dates",
    )

    def save(self, *args, **kwargs):
        complications = [
            "stroke",
            "heart_attack",
            "renal_disease",
            "vision",
            "numbness",
            "foot_ulcers",
        ]
        for complication in complications:
            if getattr(self, f"{complication}_ago", None):
                duration = duration_to_date(
                    getattr(self, f"{complication}_ago", None), self.report_datetime
                )
                setattr(self, f"{complication}_estimated_date", duration)

        super().save(*args, **kwargs)

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Complications: Baseline"
        verbose_name_plural = "Complications: Baseline"



# Generated by Django 4.2.3 on 2023-07-18 01:35

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0057_rename_smoker_current_duration_historicalotherbaselinedata_smoker_duration_and_more",
        ),
    ]

    operations = [
        migrations.RenameField(
            model_name="healtheconomicsincome",
            old_name="external_remittance_currency",
            new_name="external_remit_currency",
        ),
        migrations.RenameField(
            model_name="healtheconomicsincome",
            old_name="external_remittance_currency_other",
            new_name="external_remit_currency_other",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicsincome",
            old_name="external_remittance_currency",
            new_name="external_remit_currency",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicsincome",
            old_name="external_remittance_currency_other",
            new_name="external_remit_currency_other",
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-21 18:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_screening",
            "0002_remove_healthtalklog_unique_lower_name_report_date_and_more",
        ),
        ("intecomm_group", "0003_alter_historicalpatientgroup_group_identifier_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="patientgroup",
            name="patients",
            field=models.ManyToManyField(
                blank=True, to="intecomm_screening.patientlog", verbose_name="Patients"
            ),
        ),
    ]



from django.contrib import admin
from django.utils.html import format_html
from django_audit_fields import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_crf.modeladmin_mixins import CrfStatusModelAdminMixin
from edc_model_admin.dashboard import ModelAdminCrfDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_model_admin.mixins import ModelAdminProtectPiiMixin
from edc_sites.admin import SiteModelAdminMixin

medication_adherence_description = """
<H5><B><font color="orange">Interviewer to read</font></B></H5>
<p>Drag the slider on the line below at
the point showing your best guess about how much medication
you have taken in the last 28 days:<BR><BR>
<B>0%</B> means you have taken no medication<BR>
<B>50%</B> means you have taken half of your medication<BR>
<B>100%</B> means you have taken all your medication<BR>
</p>
"""


class CrfModelAdmin(
    ModelAdminProtectPiiMixin,  # must remain first
    SiteModelAdminMixin,
    CrfStatusModelAdminMixin,
    ModelAdminCrfDashboardMixin,
    SimpleHistoryAdmin,
):
    pass


class MedicationAdherenceAdminMixin:
    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Visual Score",
            {
                "description": format_html(medication_adherence_description),
                "fields": ("visual_score_slider", "visual_score_confirmed"),
            },
        ),
        (
            "Missed Medications",
            {
                "fields": (
                    "last_missed_pill",
                    "missed_pill_reason",
                    "other_missed_pill_reason",
                )
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    filter_horizontal = ["missed_pill_reason"]

    radio_fields = {
        "last_missed_pill": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
    }



from bs4 import BeautifulSoup
from django import template
from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse
from edc_constants.constants import NO, TBD, YES
from edc_dashboard.url_names import url_names
from edc_dashboard.utils import get_bootstrap_version
from edc_screening.constants import ELIGIBLE, NOT_ELIGIBLE

from intecomm_screening.models import PatientLog
from intecomm_screening.utils import get_consent_refusal_url, get_subject_consent_url

register = template.Library()


@register.inclusion_tag(
    f"intecomm_dashboard/bootstrap{get_bootstrap_version()}/" f"buttons/screening_button.html",
    takes_context=True,
)
def screening_button(context, model_wrapper):
    title = "Edit subject's screening form"
    perms = context["perms"]
    if model_wrapper.object.eligible is False:
        eligible = NO
    elif model_wrapper.object.eligible is True:
        eligible = YES
    else:
        eligible = TBD
    enabled = perms.user.has_perm(
        "intecomm_screening.view_subjectscreening"
    ) or perms.user.has_perm("intecomm_screening.change_subjectscreening")
    return dict(
        perms=context["perms"],
        screening_identifier=model_wrapper.object.screening_identifier,
        enabled=enabled,
        eligible=eligible,
        title=title,
        YES=YES,
        NO=NO,
        TBD=TBD,
        href=model_wrapper.href,
    )


@register.inclusion_tag(
    f"intecomm_dashboard/bootstrap{get_bootstrap_version()}/"
    f"buttons/eligibility_button.html"
)
def eligibility_button(subject_screening_model_wrapper):
    comment = []
    obj = subject_screening_model_wrapper.object
    tooltip = None
    if obj.reasons_ineligible:
        comment = obj.reasons_ineligible.split("|")
        comment = list(set(comment))
        comment.sort()
    display_label = ELIGIBLE if obj.eligible else NOT_ELIGIBLE
    soup = BeautifulSoup(display_label, features="html.parser")
    return dict(
        eligible=obj.eligible,
        eligible_final=obj.eligible,
        display_label=soup.get_text(),
        comment=comment,
        tooltip=tooltip,
        TBD=TBD,
    )


@register.inclusion_tag(
    f"intecomm_dashboard/bootstrap{get_bootstrap_version()}/buttons/add_consent_button.html",
    takes_context=True,
)
def add_consent_button(context, model_wrapper):
    title = ["Consent subject to participate."]
    consent_version = model_wrapper.consent.version
    url = get_subject_consent_url(
        subject_screening=model_wrapper.object,
        next_url_name="intecomm_dashboard:screening_listboard_url,screening_identifier",
    )
    return dict(
        perms=context["perms"],
        screening_identifier=model_wrapper.object.screening_identifier,
        href=model_wrapper.consent.href,
        consent_version=consent_version,
        title=" ".join(title),
        add_consent_url=url,
        change_consent_url=url,
        subject_identifier=model_wrapper.object.subject_identifier,
    )


@register.inclusion_tag(
    f"intecomm_dashboard/bootstrap{get_bootstrap_version()}/"
    f"buttons/patient_log_button.html",
    takes_context=True,
)
def patient_log_button(context, model_wrapper):
    change_list_href = None
    title = "Go to patient's log"
    screening_identifier = model_wrapper.object.screening_identifier
    try:
        obj = PatientLog.objects.get(screening_identifier=screening_identifier)
    except ObjectDoesNotExist:
        pass
    else:
        change_list_href = reverse(
            "intecomm_screening_admin:intecomm_screening_patientlog_changelist"
        )
        change_list_href = f"{change_list_href}?q={obj.id}"
    return dict(
        perms=context["perms"],
        change_list_href=change_list_href,
        title=title,
    )


@register.inclusion_tag(
    f"intecomm_dashboard/bootstrap{get_bootstrap_version()}/"
    f"buttons/patient_group_button.html",
    takes_context=True,
)
def patient_group_button(context, model_wrapper):
    change_list_href = None
    patient_group = None
    title = "Go to patient's group"
    screening_identifier = model_wrapper.object.screening_identifier
    try:
        obj = PatientLog.objects.get(screening_identifier=screening_identifier)
    except ObjectDoesNotExist:
        pass
    else:
        if patient_group := obj.patientgroup_set.all().first():
            change_list_href = reverse(
                "intecomm_screening_admin:intecomm_screening_patientgroup_changelist"
            )
            change_list_href = f"{change_list_href}?q={patient_group.name}"
    return dict(
        perms=context["perms"],
        change_list_href=change_list_href,
        title=title,
        patient_group=patient_group,
    )


@register.inclusion_tag(
    f"intecomm_dashboard/bootstrap{get_bootstrap_version()}/" f"buttons/refusal_button.html",
    takes_context=True,
)
def refusal_button(context, model_wrapper):
    title = ["Capture patient's primary reason for not consenting."]
    subject_screening = model_wrapper.object
    url = get_consent_refusal_url(screening_identifier=subject_screening.screening_identifier)

    return dict(
        perms=context["perms"],
        href=url,
        title=" ".join(title),
    )


@register.inclusion_tag(
    f"intecomm_dashboard/bootstrap{get_bootstrap_version()}/buttons/dashboard_button.html"
)
def dashboard_button(model_wrapper):
    subject_dashboard_url = url_names.get("subject_dashboard_url")
    subject_dashboard_href = reverse(
        subject_dashboard_url, args=(model_wrapper.subject_identifier,)
    )
    title = "Go to subject's dashboard"
    return dict(
        subject_dashboard_href=subject_dashboard_href,
        subject_identifier=model_wrapper.subject_identifier,
        title=title,
    )


@register.inclusion_tag(
    f"intecomm_dashboard/bootstrap{get_bootstrap_version()}/changelist_topbar.html",
    takes_context=True,
)
def intecomm_changelist_topbar(context, selected: str):
    context["selected"] = selected
    return context



from django.db import models
from edc_model.models import BaseUuidModel
from edc_rx.model_mixins import DrugSupplyModelMixin

from intecomm_lists.models import DmTreatments

from .drug_refill_dm import DrugRefillDm


class DrugSupplyDm(DrugSupplyModelMixin, BaseUuidModel):
    drug_refill = models.ForeignKey(DrugRefillDm, on_delete=models.PROTECT)

    drug = models.ForeignKey(DmTreatments, on_delete=models.PROTECT)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Drug Supply: Diabetes"
        verbose_name_plural = "Drug Supply: Diabetes"



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_model_fields.widgets import SliderWidget
from intecomm_form_validators.subject import HivMedicationAdherenceFormValidator

from ..models import HivMedicationAdherence


class HivMedicationAdherenceForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = HivMedicationAdherenceFormValidator

    visual_score_slider = forms.CharField(
        label="Visual Score", widget=SliderWidget(attrs={"min": 0, "max": 100})
    )

    class Meta:
        model = HivMedicationAdherence
        fields = "__all__"



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item.fieldsets import action_fieldset_tuple
from edc_adverse_event.modeladmin_mixins import DeathReportModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_ae_admin
from ..forms import DeathReportForm
from ..models import DeathReport


@admin.register(DeathReport, site=intecomm_ae_admin)
class DeathReportAdmin(SiteModelAdminMixin, DeathReportModelAdminMixin, SimpleHistoryAdmin):
    form = DeathReportForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "report_datetime",
                    "death_datetime",
                )
            },
        ),
        (
            "Location",
            {
                "fields": (
                    "death_as_inpatient",
                    "death_location",
                    "hospital_name",
                )
            },
        ),
        (
            "Informant Information",
            {
                "fields": (
                    "informant_contact",
                    "informant_relationship",
                    "other_informant_relationship",
                )
            },
        ),
        (
            "Cause of death",
            {
                "description": (
                    "If death occurred in hospital or a death certificate is available, "
                    "please indicate the recorded causes of death"
                ),
                "fields": (
                    "death_certificate",
                    "cause_of_death",
                    "narrative",
                ),
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "death_as_inpatient": admin.VERTICAL,
        "death_location": admin.VERTICAL,
        "informant_relationship": admin.VERTICAL,
        "death_certificate": admin.VERTICAL,
    }



from django import forms
from edc_crf.modelform_mixins import CrfSingletonModelFormMixin
from edc_he.form_validators import HealthEconomicsAssetsFormValidator

from ...models import HealthEconomicsAssets
from ..mixins import CrfModelFormMixin
from .modelform_mixins import HealthEconomicsModelFormMixin


class HealthEconomicsAssetsForm(
    CrfSingletonModelFormMixin,
    HealthEconomicsModelFormMixin,
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = HealthEconomicsAssetsFormValidator

    class Meta:
        model = HealthEconomicsAssets
        fields = "__all__"



# Generated by Django 4.2.3 on 2023-08-03 00:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("edc_he", "0003_ethnicities_extra_value_and_more"),
        ("intecomm_subject", "0070_healtheconomicshouseholdhead_hoh_ethnicity_old_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_ethnicity",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_he.ethnicities",
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_ethnicity",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_he.ethnicities",
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_ethnicity_old",
            field=models.CharField(
                choices=[
                    ("baganda", "Baganda"),
                    ("banyakole", "Banyakole"),
                    ("basoga", "Basoga"),
                    ("bakiga", "Bakiga"),
                    ("iteso", "Iteso"),
                    ("langi", "Langi"),
                    ("acholi", "Acholi"),
                    ("bagisu", "Bagisu"),
                    ("lugbara", "Lugbara"),
                    ("banyoro", "Banyoro"),
                    ("bakhonzo", "Bakhonzo"),
                    ("batoro", "Batoro"),
                    ("alur", "Alur"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_ethnicity_old",
            field=models.CharField(
                choices=[
                    ("baganda", "Baganda"),
                    ("banyakole", "Banyakole"),
                    ("basoga", "Basoga"),
                    ("bakiga", "Bakiga"),
                    ("iteso", "Iteso"),
                    ("langi", "Langi"),
                    ("acholi", "Acholi"),
                    ("bagisu", "Bagisu"),
                    ("lugbara", "Lugbara"),
                    ("banyoro", "Banyoro"),
                    ("bakhonzo", "Bakhonzo"),
                    ("batoro", "Batoro"),
                    ("alur", "Alur"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
    ]



# Generated by Django 4.1.7 on 2023-04-04 00:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_lists", "0005_htnmanagement_and_more"),
        ("intecomm_subject", "0020_remove_historicalhtninitialreview_managed_by_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="htninitialreview",
            name="managed_by",
            field=models.ManyToManyField(
                to="intecomm_lists.htnmanagement",
                verbose_name="How is the patient's hypertension managed?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhtninitialreview",
            name="managed_by_old",
            field=models.CharField(
                choices=[
                    ("drugs", "Drugs / Medicine"),
                    ("diet_lifestyle", "Diet and lifestyle alone"),
                ],
                default="N/A",
                help_text="If patient is taking medication, complete the Drug Refill CRF.",
                max_length=15,
                verbose_name="How is the patient's hypertension managed?",
            ),
        ),
        migrations.AlterField(
            model_name="htninitialreview",
            name="managed_by_old",
            field=models.CharField(
                choices=[
                    ("drugs", "Drugs / Medicine"),
                    ("diet_lifestyle", "Diet and lifestyle alone"),
                ],
                default="N/A",
                help_text="If patient is taking medication, complete the Drug Refill CRF.",
                max_length=15,
                verbose_name="How is the patient's hypertension managed?",
            ),
        ),
    ]



from edc_model.models import BaseUuidModel
from edc_qol.model_mixins import IcecapaModelMixin

from ..model_mixins import CrfModelMixin


class Icecapa(
    IcecapaModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):
    class Meta(IcecapaModelMixin.Meta, CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Overall quality of life (ICECAP-A V2)"
        verbose_name_plural = "Overall quality of life (ICECAP-A V2)"



# create next appt using facility appt date
from datetime import datetime
from zoneinfo import ZoneInfo

from django.conf import settings
from django.db.models.signals import post_save
from django.dispatch import receiver
from edc_appointment.constants import (
    COMPLETE_APPT,
    IN_PROGRESS_APPT,
    MISSED_APPT,
    NEW_APPT,
)
from edc_constants.constants import ALIVE, OTHER
from edc_utils import to_utc

from intecomm_lists.models import SubjectVisitMissedReasons

from ..constants import NOT_SCHEDULED_FOR_FACILITY
from .next_appointment import NextAppointment
from .subject_visit import SubjectVisit
from .subject_visit_missed import SubjectVisitMissed


def update_appointments(instance):
    appointment = instance.subject_visit.appointment.next
    while appointment:
        if appointment.appt_status != NEW_APPT:
            break
        if (
            appointment.visit_code == instance.best_visit_code
            and appointment.visit_code_sequence == 0
        ):
            appointment.appt_datetime = to_utc(
                datetime(
                    instance.appt_date.year,
                    month=instance.appt_date.month,
                    day=instance.appt_date.day,
                    hour=8,
                    minute=0,
                    tzinfo=ZoneInfo(settings.TIME_ZONE),
                )
            )
            appointment.save(update_fields=["appt_datetime"])
            break
        else:
            appointment.appt_status = IN_PROGRESS_APPT
            appointment.appt_timing = MISSED_APPT
            appointment.save(update_fields=["appt_status", "appt_timing"])

            subject_visit = SubjectVisit.objects.get(appointment=appointment)
            subject_visit.report_datetime = appointment.appt_datetime
            subject_visit.save()
            subject_visit_missed = SubjectVisitMissed.objects.create(
                subject_visit=subject_visit,
                survival_status=ALIVE,
                report_datetime=appointment.appt_datetime,
                contact_last_date=None,
            )
            subject_visit_missed.missed_reasons.add(
                SubjectVisitMissedReasons.objects.get(name=OTHER)
            )
            subject_visit_missed.missed_reasons_other = NOT_SCHEDULED_FOR_FACILITY
            subject_visit_missed.comment = "[auto-completed by EDC]"
            subject_visit_missed.save()
            appointment.appt_status = COMPLETE_APPT
            appointment.save(update_fields=["appt_status"])
        appointment = appointment.next


@receiver(
    post_save,
    weak=False,
    sender=NextAppointment,
    dispatch_uid="update_next_appointment_on_post_save",
)
def update_next_appointment_on_post_save(sender, instance, raw, created, using, **kwargs):
    if not raw and not kwargs.get("update_fields"):
        if instance.appt_date and instance.best_visit_code:
            update_appointments(instance)



from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from edc_registration.models import RegisteredSubject

from intecomm_screening.models import SubjectScreening

from .subject_consent import SubjectConsent


class SubjectConsentDeleteError(Exception):
    pass


@receiver(
    post_save,
    weak=False,
    sender=SubjectConsent,
    dispatch_uid="subject_consent_on_post_save",
)
def subject_consent_on_post_save(sender, instance, raw, created, **kwargs):
    if not raw:
        if created:
            subject_screening = SubjectScreening.objects.get(
                screening_identifier=instance.screening_identifier
            )
            subject_screening.subject_identifier = instance.subject_identifier
            subject_screening.consented = True
            subject_screening.save_base(update_fields=["subject_identifier", "consented"])

            subject_screening.patient_log.subject_identifier = instance.subject_identifier
            subject_screening.patient_log.consent_datetime = instance.consent_datetime
            subject_screening.patient_log.save_base(
                update_fields=["subject_identifier", "consent_datetime"]
            )
            # TODO: is this the right place to do this?
            rs_obj = RegisteredSubject.objects.get(
                subject_identifier=instance.subject_identifier
            )
            try:
                rs_obj.full_name = instance.full_name
            except AttributeError:
                pass
            try:
                rs_obj.familiar_name = instance.familiar_name
            except AttributeError:
                pass
            rs_obj.save(update_fields=["full_name", "familiar_name"])


@receiver(
    pre_delete,
    weak=False,
    sender=SubjectConsent,
    dispatch_uid="subject_consent_on_pre_delete",
)
def subject_consent_on_pre_delete(sender, instance, using, **kwargs):
    raise SubjectConsentDeleteError(
        f"Subject consent may not be deleted. Got {instance.subject_identifier}."
    )



from django.contrib import admin
from edc_sites import get_current_country

from ..admin_site import intecomm_screening_admin
from ..models import Site


@admin.register(Site, site=intecomm_screening_admin)
class SiteAdmin(admin.ModelAdmin):
    search_fields = ("id", "name")

    def get_queryset(self, request):
        country = get_current_country(request)
        return self.model.objects.filter(siteprofile__country=country).order_by("name")



# Generated by Django 4.2.1 on 2023-06-30 22:17
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from django.db.migrations import RunPython
from edc_constants.constants import NO, NOT_APPLICABLE, TBD, YES
from tqdm import tqdm


def update_willing_to_screen(apps, schema_editor):
    patientlog_cls = apps.get_model("intecomm_screening.patientlog")
    subjectscreening_cls = apps.get_model("intecomm_screening.subjectscreening")
    screeningrefusalreasons_cls = apps.get_model("intecomm_lists.screeningrefusalreasons")
    # may fail here if post-migrate needs to run
    not_applicable = screeningrefusalreasons_cls.objects.get(name=NOT_APPLICABLE)
    qs = patientlog_cls.objects.all()
    for patientlog in tqdm(qs, total=qs.count()):
        try:
            subjectscreening_cls.objects.get(patient_log_id=patientlog.id)
        except ObjectDoesNotExist:
            if patientlog.screening_refusal_reason:
                patientlog.willing_to_screen = NO
            else:
                patientlog.willing_to_screen = TBD
                patientlog.screening_refusal_reason = not_applicable
        else:
            patientlog.willing_to_screen = YES
            patientlog.screening_refusal_reason = not_applicable
        patientlog.save_base(update_fields=["willing_to_screen", "screening_refusal_reason"])


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0037_alter_consentrefusal_screening_identifier_and_more"),
    ]

    operations = [RunPython(update_willing_to_screen)]



from django import forms
from edc_action_item.forms import ActionItemCrfFormMixin
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_lab_panel.panels import blood_glucose_panel
from edc_lab_results.form_validator_mixins import (
    BloodResultsFormValidatorMixin,
    BloodResultsGluFormValidatorMixin,
)

from ...models import BloodResultsGlu


class BloodResultsGluFormValidator(
    BloodResultsGluFormValidatorMixin, BloodResultsFormValidatorMixin, CrfFormValidator
):
    panel = blood_glucose_panel


class BloodResultsGluForm(ActionItemCrfFormMixin, CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = BloodResultsGluFormValidator

    class Meta(ActionItemCrfFormMixin.Meta):
        model = BloodResultsGlu
        fields = "__all__"



# Generated by Django 4.2.3 on 2023-07-25 21:07

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_facility", "0002_auto_20230726_2030"),
        ("intecomm_subject", "0064_historicalnextappointment_health_facility_old_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalnextappointment",
            name="old_health_facility",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name="nextappointment",
            name="old_health_facility",
            field=models.CharField(max_length=100, null=True),
        ),
    ]



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from django.utils.safestring import mark_safe
from edc_action_item import ActionWithNotification, site_action_items
from edc_adverse_event.action_items import HospitalizationAction
from edc_adverse_event.constants import (
    AE_FOLLOWUP_ACTION,
    AE_INITIAL_ACTION,
    DEATH_REPORT_ACTION,
)
from edc_constants.constants import DEAD, HIGH_PRIORITY, YES
from edc_ltfu.constants import LOST_TO_FOLLOWUP
from edc_notification.utils import get_email_contacts
from edc_reportable import GRADE5
from edc_visit_schedule.utils import get_offschedule_models
from intecomm_rando.constants import CLINIC_CONTROL, COMM_INTERVENTION

from intecomm_group.utils import get_assignment_for_patient_group
from intecomm_prn.constants import OFFSCHEDULE_COMM_ACTION, OFFSCHEDULE_INTE_ACTION

if TYPE_CHECKING:
    from intecomm_prn.models import OffScheduleComm, OffScheduleInte


class AeFollowupAction(ActionWithNotification):
    name = AE_FOLLOWUP_ACTION
    display_name = "Submit AE Followup Report"
    notification_display_name = "AE Followup Report"
    parent_action_names = [AE_INITIAL_ACTION, AE_FOLLOWUP_ACTION]
    reference_model = "intecomm_ae.aefollowup"
    related_reference_model = "intecomm_ae.aeinitial"
    related_reference_fk_attr = "ae_initial"
    create_by_user = False
    show_link_to_changelist = True
    admin_site_name = "intecomm_ae_admin"
    instructions = mark_safe(  # nosec B308, B703
        "Upon submission the TMG group will be notified "
        f'by email at <a href="mailto:{get_email_contacts("tmg") or "#"}">'
        f'{get_email_contacts("tmg") or "unknown"}</a>'
    )
    priority = HIGH_PRIORITY

    def get_next_actions(self):
        next_actions = []

        # add AE followup to next_actions if followup.
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=self.name,
            required=self.reference_obj.followup == YES,
        )

        # add Death Report to next_actions if G5/Death
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=DEATH_REPORT_ACTION,
            required=(
                self.reference_obj.outcome == DEAD or self.reference_obj.ae_grade == GRADE5
            ),
        )

        # add Study termination to next_actions if LTFU
        if self.reference_obj.outcome == LOST_TO_FOLLOWUP:
            for offschedule_model in get_offschedule_models(
                subject_identifier=self.subject_identifier,
                report_datetime=self.reference_obj.report_datetime,
            ):
                action_cls = site_action_items.get_by_model(model=offschedule_model)
                next_actions = self.append_to_next_if_required(
                    next_actions=next_actions,
                    action_name=action_cls.name,
                    required=True,
                )
        return next_actions


class AeInitialAction(ActionWithNotification):
    name = AE_INITIAL_ACTION
    display_name = "Submit AE Initial Report"
    notification_display_name = "AE Initial Report"
    reference_model = "intecomm_ae.aeinitial"
    show_link_to_changelist = True
    show_link_to_add = True
    admin_site_name = "intecomm_ae_admin"
    instructions = "Complete the initial AE report"
    priority = HIGH_PRIORITY

    def get_next_actions(self):
        """Returns next actions.

        1. Add death report action if death
        """
        next_actions = []
        deceased = self.reference_obj.ae_grade == GRADE5
        # add next AeFollowup if not deceased
        if not deceased:
            next_actions = self.append_to_next_if_required(
                action_name=AE_FOLLOWUP_ACTION, next_actions=next_actions
            )
        # add next Death report if G5/Death
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=DEATH_REPORT_ACTION,
            required=deceased,
        )
        return next_actions


class DeathReportAction(ActionWithNotification):
    name = DEATH_REPORT_ACTION
    display_name = "Submit Death Report"
    notification_display_name = "Death Report"
    reference_model = "intecomm_ae.deathreport"
    parent_action_names = [AE_INITIAL_ACTION, AE_FOLLOWUP_ACTION]
    show_link_to_changelist = True
    show_link_to_add = True
    admin_site_name = "intecomm_ae_admin"
    priority = HIGH_PRIORITY
    singleton = True
    dirty_fields = ["cause_of_death"]

    def get_next_actions(self):
        next_actions = []
        try:
            self.off_schedule_cls.objects.get(subject_identifier=self.subject_identifier)
        except ObjectDoesNotExist:
            if self.assignment == COMM_INTERVENTION:
                next_actions.extend([OFFSCHEDULE_COMM_ACTION])
            elif self.assignment == CLINIC_CONTROL:
                next_actions.extend([OFFSCHEDULE_INTE_ACTION])
        return next_actions

    @property
    def group_identifier(self) -> str:
        return (
            django_apps.get_model("intecomm_screening.patientlog")
            .objects.get(subject_identifier=self.subject_identifier)
            .group_identifier
        )

    @property
    def assignment(self) -> str:
        return get_assignment_for_patient_group(self.group_identifier)

    @property
    def off_schedule_cls(self) -> OffScheduleComm | OffScheduleInte:
        if self.assignment == COMM_INTERVENTION:
            return django_apps.get_model("intecomm_prn.offschedulecomm")
        elif self.assignment == CLINIC_CONTROL:
            return django_apps.get_model("intecomm_prn.offscheduleinte")


class HospitalizationAction(HospitalizationAction):
    reference_model = "intecomm_ae.hospitalization"
    admin_site_name = "intecomm_ae_admin"


site_action_items.register(AeFollowupAction)
site_action_items.register(AeInitialAction)
site_action_items.register(DeathReportAction)
site_action_items.register(HospitalizationAction)



from django.contrib import admin
from edc_lab_results.fieldsets import BloodResultFieldset

from ..admin_site import intecomm_subject_admin
from ..forms import Cd4ResultForm
from ..models import Cd4Result
from .modeladmin_mixins import CrfModelAdmin


@admin.register(Cd4Result, site=intecomm_subject_admin)
class Cd4ResultAdmin(CrfModelAdmin):
    form = Cd4ResultForm
    autocomplete_fields = ["requisition"]
    fieldsets = BloodResultFieldset(
        Cd4Result.lab_panel,
        model_cls=Cd4Result,
        exclude_units=True,
        exclude_reportable=True,
    ).fieldsets



# Generated by Django 4.2.1 on 2023-05-23 23:18

from django.db import migrations, models
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0043_healtheconomicshouseholdhead_relationship_to_hoh_other_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_education_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER education, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_ethnicity_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER ethnic background, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_insurance_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True, max_length=35, null=True, verbose_name="If OTHER, specify ..."
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_religion_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER religious orientation, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_education_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER education, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_ethnicity_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER ethnic background, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_insurance_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True, max_length=35, null=True, verbose_name="If OTHER, specify ..."
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_religion_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER religious orientation, specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_citizen",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Is the household head a citizen of this country?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_type",
            field=models.CharField(
                choices=[
                    ("1", "1. Legislators, administrators, and managers"),
                    ("2", "2. Professionals"),
                    ("3", "3. Technicians and associate professionals"),
                    ("4", "4. Clerks"),
                    ("5", "5. Service workers and shop sale workers"),
                    ("6", "6. Skilled agricultural and fishery workers"),
                    ("7", "7. Unskilled agricultural and fishery workers"),
                    ("8", "8. Craft and related workers"),
                    ("9", "9. Plant and machine operators and assemblers"),
                    ("10", "10. Elementary occupations"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "1. Never Married (but not co-habiting)"),
                    ("2", "2. Co-habiting"),
                    ("3", "3. Currently Married"),
                    ("4", "4. Separated/Divorced"),
                    ("5", "5. Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Household head’s marital status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_citizen",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Is the patient a citizen of this country?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_employment_type",
            field=models.CharField(
                choices=[
                    ("1", "1. Legislators, administrators, and managers"),
                    ("2", "2. Professionals"),
                    ("3", "3. Technicians and associate professionals"),
                    ("4", "4. Clerks"),
                    ("5", "5. Service workers and shop sale workers"),
                    ("6", "6. Skilled agricultural and fishery workers"),
                    ("7", "7. Unskilled agricultural and fishery workers"),
                    ("8", "8. Craft and related workers"),
                    ("9", "9. Plant and machine operators and assemblers"),
                    ("10", "10. Elementary occupations"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your type of employment?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "1. Never Married (but not co-habiting)"),
                    ("2", "2. Co-habiting"),
                    ("3", "3. Currently Married"),
                    ("4", "4. Separated/Divorced"),
                    ("5", "5. Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your marital status?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_citizen",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Is the household head a citizen of this country?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_type",
            field=models.CharField(
                choices=[
                    ("1", "1. Legislators, administrators, and managers"),
                    ("2", "2. Professionals"),
                    ("3", "3. Technicians and associate professionals"),
                    ("4", "4. Clerks"),
                    ("5", "5. Service workers and shop sale workers"),
                    ("6", "6. Skilled agricultural and fishery workers"),
                    ("7", "7. Unskilled agricultural and fishery workers"),
                    ("8", "8. Craft and related workers"),
                    ("9", "9. Plant and machine operators and assemblers"),
                    ("10", "10. Elementary occupations"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "1. Never Married (but not co-habiting)"),
                    ("2", "2. Co-habiting"),
                    ("3", "3. Currently Married"),
                    ("4", "4. Separated/Divorced"),
                    ("5", "5. Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Household head’s marital status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_citizen",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Is the patient a citizen of this country?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_employment_type",
            field=models.CharField(
                choices=[
                    ("1", "1. Legislators, administrators, and managers"),
                    ("2", "2. Professionals"),
                    ("3", "3. Technicians and associate professionals"),
                    ("4", "4. Clerks"),
                    ("5", "5. Service workers and shop sale workers"),
                    ("6", "6. Skilled agricultural and fishery workers"),
                    ("7", "7. Unskilled agricultural and fishery workers"),
                    ("8", "8. Craft and related workers"),
                    ("9", "9. Plant and machine operators and assemblers"),
                    ("10", "10. Elementary occupations"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your type of employment?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "1. Never Married (but not co-habiting)"),
                    ("2", "2. Co-habiting"),
                    ("3", "3. Currently Married"),
                    ("4", "4. Separated/Divorced"),
                    ("5", "5. Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your marital status?",
            ),
        ),
    ]



from django import forms
from edc_constants.constants import HIV
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx.form_validators import ResultFormValidatorMixin
from edc_form_validators.form_validator import FormValidator

from ..models import ViralLoadResult


class ViralLoadResultFormValidator(
    ResultFormValidatorMixin, CrfFormValidatorMixin, FormValidator
):
    dx = (HIV, "HIV infection")


class ViralLoadResultForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = ViralLoadResultFormValidator

    class Meta:
        model = ViralLoadResult
        fields = "__all__"



# Generated by Django 4.1.7 on 2023-04-19 02:28

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0018_alter_historicalpatientlog_age_in_years_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="healthfacility",
            name="fri",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="healthfacility",
            name="mon",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="healthfacility",
            name="sat",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="healthfacility",
            name="thu",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="healthfacility",
            name="tue",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="healthfacility",
            name="wed",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalhealthfacility",
            name="fri",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalhealthfacility",
            name="mon",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalhealthfacility",
            name="sat",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalhealthfacility",
            name="thu",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalhealthfacility",
            name="tue",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalhealthfacility",
            name="wed",
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-22 20:31

import django.core.validators
from django.db import migrations
import django_crypto_fields.fields.encrypted_char_field


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_screening",
            "0008_rename_last_routine_appt_date_historicalpatientlog_last_appt_date_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="familiar_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="How should we refer to you? (if we speak to you)",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="legal_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                db_index=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure full name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="Full name",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="familiar_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="How should we refer to you? (if we speak to you)",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="legal_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure full name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="Full name",
            ),
        ),
    ]



from edc_constants.constants import (
    CLINICAL_WITHDRAWAL,
    COMMUNITY,
    DEAD,
    DM,
    DONT_KNOW,
    DWTA,
    ESTIMATED,
    FREE_OF_CHARGE,
    HIV,
    HTN,
    INSURANCE,
    NONE,
    NOT_APPLICABLE,
    OTHER,
    OWN_CASH,
    PATIENT,
    PATIENT_CLUB,
    PATIENT_REPRESENTATIVE,
    PREGNANCY,
    PRIVATE,
    RELATIVE,
)
from edc_dx_review.constants import DIET_LIFESTYLE, DRUGS, INSULIN
from edc_he.constants import CHF, NHIF
from edc_list_data.row import Row
from edc_ltfu.constants import LOST_TO_FOLLOWUP
from edc_offstudy.constants import COMPLETED_FOLLOWUP, CONSENT_WITHDRAWAL
from edc_transfer.constants import TRANSFERRED

from intecomm_lists.constants import HEALTH_RECORDS
from intecomm_subject.constants import VIT_B_FOLIC_ACID

list_data = {
    "intecomm_lists.drugpaysources": [
        (OWN_CASH, "Own cash"),
        (INSURANCE, "Insurance"),
        (PATIENT_CLUB, "Patient support group / club"),
        (RELATIVE, "Relative or others paying"),
        (FREE_OF_CHARGE, "Free drugs from the pharmacy"),
        (OTHER, "Other pay source (specify below)"),
    ],
    "intecomm_lists.offstudyreasons": [
        (COMPLETED_FOLLOWUP, "Patient completed 12 months of follow-up"),
        (CONSENT_WITHDRAWAL, "Patient withdrew consent to participate further"),
        (CLINICAL_WITHDRAWAL, "Patient is withdrawn on CLINICAL grounds ..."),
        (LOST_TO_FOLLOWUP, "Patient lost to follow-up (at 12m only)"),
        (
            TRANSFERRED,
            "Patient has been transferred to another health centre/ area (at 12m only)",
        ),
        (DEAD, "Patient reported/known to have died"),
        (
            OTHER,
            "Other reason (specify below)",
        ),
    ],
    "intecomm_lists.subjectvisitmissedreasons": [
        ("forgot", "Forgot / Can’t remember being told about appointment"),
        ("family_emergency", "Family emergency (e.g. funeral) and was away"),
        ("travelling", "Away travelling/visiting"),
        ("working_schooling", "Away working/schooling"),
        ("too_sick", "Too sick or weak to come to the centre"),
        ("lack_of_transport", "Transportation difficulty"),
        (OTHER, "Other reason (specify below)"),
    ],
    "intecomm_lists.clinicalwithdrawalreasons": [
        (PREGNANCY, "Pregnancy"),
        ("kidney_disease", "Development of chronic kidney disease"),
        ("liver_disease", "Development of chronic liver disease"),
        ("intercurrent_illness", "Intercurrent illness which prevents further treatment"),
        ("investigator_decision", "Investigator decision"),
        (NOT_APPLICABLE, "Not applicable"),
        (
            OTHER,
            (
                "Other condition that justifies the discontinuation of "
                "treatment in the clinician’s opinion (specify below)"
            ),
        ),
    ],
    "intecomm_lists.healthtalktypes": [
        ("study_intro_talk", "INTECOMM study introduction"),
        ("sensitisation_talk", "INTECOMM Sensitisation"),
        (OTHER, "Other (specify below)"),
    ],
    "intecomm_lists.conditions": [
        (HIV, "HIV"),
        (DM, "Diabetes"),
        (HTN, "Hypertension"),
    ],
    "intecomm_lists.locationtypes": [
        ("school", "School"),
        (OTHER, "Other (specify below)"),
    ],
    "intecomm_lists.dmmanagement": [
        (INSULIN, "Insulin injections"),
        (DRUGS, "Oral drugs"),
        (DIET_LIFESTYLE, "Diet and lifestyle"),
        (OTHER, "Other, please specify below ..."),
    ],
    "intecomm_lists.htnmanagement": [
        (DRUGS, "Oral drugs"),
        (DIET_LIFESTYLE, "Diet and lifestyle"),
        (OTHER, "Other, please specify below ..."),
    ],
    "intecomm_lists.htntreatments": [
        ("aldactone", "Aldactone (Spironolactone)"),
        ("amlodipine", "Amlodipine"),
        ("atenolol", "Atenolol"),
        ("atorvastatin", "Atorvastatin"),
        ("bendroflumethiazide", "Bendroflumethiazide"),
        ("bisoprolol", "Bisoprolol"),
        ("candesartan", "Candesartan"),
        ("captopril", "Captopril"),
        ("carvedilol", "Carvedilol"),
        ("clopidogrel", "Clopidogrel"),
        ("enalapril", "Enalapril"),
        ("frusemide", "Frusemide"),
        ("hydralazine", "Hydralazine"),
        ("hydrochlorothiazide", "Hydrochlorothiazide"),
        ("irbesartan", "Irbesartan"),
        ("irbesartan_hydrochlorothiazide", "Irbesartan Hydrochlorothiazide"),
        ("junior_aspirin", "Junior Aspirin"),
        ("lisinopril", "Lisinopril"),
        ("losartan_h", "losartan Hydrochlorothiazide (Losartan H/Repace H)"),
        ("losartan", "losartan"),
        ("methyldopa", "Methyldopa"),
        ("metoprolol", "Metoprolol"),
        ("nifedipine", "Nifedipine"),
        ("olmesartan", "Olmesartan"),
        ("propanolol", "Propanolol"),
        ("ramipril", "Ramipril"),
        ("rosuvastatin", "Rosuvastatin"),
        ("s-amlodipine", "S-Amlodipine"),
        ("simvastatin", "Simvastatin"),
        ("telmisartan", "Telmisartan"),
        ("valsartan", "Valsartan"),
        (VIT_B_FOLIC_ACID, "Vitamin Bs + Folic Acid"),
        (OTHER, "Other treatment (specify below)"),
    ],
    "intecomm_lists.arvregimens": [
        ("ABC_3TC_ATV_r", "ABC + 3TC + ATV/r"),
        ("ABC_3TC_DTG", "ABC + 3TC + DTG"),
        ("ABC_3TC_EFV", "ABC + 3TC + EFV"),
        ("ABC_3TC_LPV_r", "ABC + 3TC + LPV/r"),
        ("AZT_3TC_ATV_r", "AZT + 3TC + ATV/r"),
        ("AZT_3TC_DTG", "AZT + 3TC + DTG"),
        ("AZT_3TC_EFV", "AZT + 3TC + EFV"),
        ("AZT_3TC_LPV_r", "AZT + 3TC + LPV/r"),
        ("AZT_3TC_NVP", "AZT + 3TC + NVP"),
        ("DTG_ABC/3TC_ATV_r", "DTG + (ABC/3TC) + ATV/r"),
        ("TDF_3TC_ATV_r", "TDF + 3TC + ATV/r"),
        ("TDF_3TC_DTG", "TDF + 3TC + DTG"),
        ("TDF_3TC_EFV", "TDF + 3TC + EFV"),
        ("TDF_3TC_LPV_r", "TDF + 3TC + LPV/r"),
        ("TDF_3TC_NVP", "TDF + 3TC + NVP"),
        ("TDF_FTC_ATV_r", "TDF + FTC + ATV/r"),
        ("TDF_FTC_DTG", "TDF + FTC + DTG"),
        ("TDF_FTC_EFV", "TDF + FTC + EFV"),
        ("TDF_FTC_LPV_r", "TDF + FTC + LPV/r"),
        ("TDF_FTC_NVP", "TDF + FTC + NVP"),
        (OTHER, "Other, specify"),
    ],
    "intecomm_lists.arvdrugs": [
        ("3TC", "3TC"),
        ("ABC", "ABC"),
        ("ATV_r", "ATV/r"),
        ("AZT", "AZT"),
        ("DTG", "DTG"),
        ("EFV", "EFV"),
        ("FTC", "FTC"),
        ("LPV_r", "LPV/r"),
        ("NVP", "NVP"),
        ("TDF", "TDF"),
        (OTHER, "Other, specify"),
    ],
    "intecomm_lists.dmtreatments": [
        ("glibenclamide_metformin", "Glibenclamide + Metformin combo"),
        ("glibenclamide_s", "Glibenclamide (S)"),
        ("gliclazide_s", "Gliclazide (S)"),
        ("glimepiride_1mg_metformin", "Glimepiride (1mg) + Metformin combo"),
        ("glimepiride_2mg_metformin", "Glimepiride (2mg) + Metformin combo"),
        ("glimepiride_s", "Glimepiride (S)"),
        ("glipizide_s", "Glipizide (S)"),
        (INSULIN, "Insulin"),
        ("metformin_b", "Metformin (B)"),
        ("pioglitazone", "Pioglitazone"),
        ("pregabalin", "Pregabalin (diabetic neuropathy)"),
        (VIT_B_FOLIC_ACID, "Vitamin Bs + Folic Acid (Neuroton- diabetic neuropathy)"),
        (OTHER, "Other, specify"),
    ],
    "intecomm_lists.rxmodifications": [
        ("dose_changes", "Dose changes"),
        ("drugs_substitution", "Drugs substitution"),
        ("drug_additions", "Additional drugs added to existing regimen"),
        ("some_stopped", "Some drugs stopped"),
        (OTHER, "Other, specify"),
    ],
    "intecomm_lists.rxmodificationreasons": [
        ("availability", "Limited availability of drugs"),
        ("side_effects", "Had side-effects"),
        ("feel_better", "Felt well and stopped/reduced drug prescription"),
        (OTHER, "Other, specify"),
    ],
    "intecomm_lists.screeningrefusalreasons": [
        ("dont_have_time", "I don't have time"),
        ("must_consult_spouse", "I need to consult my spouse"),
        ("dont_want_to_join", "I don't want to take part"),
        ("need_to_think_about_it", "I haven't had a chance to think about it"),
        (NOT_APPLICABLE, "Not applicable"),
        (OTHER, "Other, specify"),
    ],
    "intecomm_lists.consentrefusalreasons": [
        ("dont_have_time", "I don't have time"),
        ("must_consult_spouse", "I need to consult my spouse"),
        ("dont_want_to_join", "I don't want to take part"),
        ("need_to_think_about_it", "I haven't had a chance to think about it"),
        (OTHER, "Other, please specify"),
    ],
    "intecomm_lists.nonadherencereasons": [
        ("forgot_to_take", "I simply forgot to take my medication"),
        ("travelled", "I travelled and forgot my medication"),
        ("feel_better", "I felt better and stopped taking my medication"),
        (
            "insufficient_supply",
            "I did not get enough medication from hospital/clinic, could not buy more",
        ),
        ("feel_ill", "The medications were making me feel sick"),
        ("too_many_pills", "Too many pills so I stopped / reduced"),
        (OTHER, "Other, please specify ..."),
    ],
    "intecomm_lists.transferreasons": [
        ("moved", "Moved away from the area"),
        ("unhappy_with_care", "Was unhappy with the care they are receiving"),
        ("need_specialized_treatment", "Needed more specialist treatment"),
        ("drug_supply", "Due to drug supply "),
        ("stigma", "Concerned about stigma"),
        (DWTA, "No reason provided"),
        (OTHER, "Other reason (specify below)"),
    ],
    "edc_he.insurancetypes": [
        (PRIVATE, "Private/work-place/voluntary health insurance"),
        Row((NHIF, "NHIF (National Health Insurance Fund)"), extra="tanzania"),
        Row((CHF, "CHF (Community Health Insurance Fund)"), extra="tanzania"),
        Row((COMMUNITY, "Community-based health insurance"), extra="uganda"),
        Row(("club", "Patient support group / club"), extra="uganda"),
        (NONE, "None"),
        (DONT_KNOW, "Don’t know"),
        (OTHER, "Other, specify"),
        (NOT_APPLICABLE, "Not applicable"),
    ],
    "edc_he.nationalities": [
        ("tanzania", "Tanzanian"),
        ("uganda", "Ugandan"),
        (NONE, "None of the above"),
    ],
    "edc_he.religions": [
        Row(("african_inland_church", "African Inland Church"), extra="tanzania"),
        ("catholic", "Catholic"),
        Row(("lutheran", "Lutheran"), extra="tanzania"),
        Row(("moravian", "Moravian"), extra="tanzania"),
        ("muslim", "Muslim"),
        ("pentecostal", "Pentecostal"),
        Row(("protestant", "Protestant"), extra="uganda"),
        ("seventh_day_adventist", "Seventh day Adventist"),
        Row(("tz_assemblies_of_god", "Tanzanian Assemblies of God"), extra="tanzania"),
        ("no_religion", "No religion"),
        (OTHER, "Other, specify"),
        (NOT_APPLICABLE, "Not applicable"),
    ],
    "edc_he.ethnicities": [
        Row(("acholi", "Acholi"), extra="uganda"),
        Row(("alur", "Alur"), extra="uganda"),
        Row(("baganda", "Baganda"), extra="uganda"),
        Row(("bagisu", "Bagisu"), extra="uganda"),
        Row(("bakhonzo", "Bakhonzo"), extra="uganda"),
        Row(("bakiga", "Bakiga"), extra="uganda"),
        Row(("banyakole", "Banyakole"), extra="uganda"),
        Row(("banyoro", "Banyoro"), extra="uganda"),
        Row(("basoga", "Basoga"), extra="uganda"),
        Row(("batoro", "Batoro"), extra="uganda"),
        Row(("iteso", "Iteso"), extra="uganda"),
        Row(("langi", "Langi"), extra="uganda"),
        Row(("lugbara", "Lugbara"), extra="uganda"),
        Row(("mchaga", "Mchaga"), extra="tanzania"),
        Row(("mhaya", "Mhaya"), extra="tanzania"),
        Row(("mhindi", "Mhindi"), extra="tanzania"),
        Row(("mjaluo", "Mjaluo"), extra="tanzania"),
        Row(("mjita", "Mjita"), extra="tanzania"),
        Row(("mkara_mkerewe", "Mkara/Mkerewe"), extra="tanzania"),
        Row(("mkuria_mshashi", "Mkuria/Mshashi"), extra="tanzania"),
        Row(("msukuma", "Msukuma"), extra="tanzania"),
        Row(("mwarabu", "Mwarabu"), extra="tanzania"),
        Row(("myiramba", "Myiramba"), extra="tanzania"),
        Row(("mzinza", "Mzinza"), extra="tanzania"),
        (OTHER, "Other, specify"),
        (NOT_APPLICABLE, "Not applicable"),
    ],
    "edc_next_appointment.infosources": [
        (COMMUNITY, "Community group"),
        (PATIENT, "Patient"),
        (
            PATIENT_REPRESENTATIVE,
            "Patient representative (e.g., next of kin, relative, guardian)",
        ),
        (HEALTH_RECORDS, "Health records"),
        (ESTIMATED, "Estimated by research staff"),
        (OTHER, "Other"),
    ],
}



from edc_crf.model_mixins import SingletonCrfModelMixin
from edc_he.model_mixins import PatientModelMixin
from edc_model.models import BaseUuidModel

from ...model_mixins import CrfModelMixin


class HealthEconomicsPatient(
    SingletonCrfModelMixin,
    PatientModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):
    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Health Economics: Patient"
        verbose_name_plural = "Health Economics: Patient"



from django.contrib import admin
from edc_list_data.admin import ListModelAdminMixin

from .admin_site import intecomm_lists_admin
from .models import Conditions, OffstudyReasons, SubjectVisitMissedReasons


@admin.register(SubjectVisitMissedReasons, site=intecomm_lists_admin)
class SubjectVisitMissedReasonsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(OffstudyReasons, site=intecomm_lists_admin)
class OffstudyReasonsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(Conditions, site=intecomm_lists_admin)
class ConditionsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass



from django.contrib import admin

from ..admin_site import intecomm_prn_admin
from ..forms import OffScheduleCommForm
from ..models import OffScheduleComm
from .modeladmin_mixins import OffScheduleAdmin


@admin.register(OffScheduleComm, site=intecomm_prn_admin)
class OffScheduleCommAdmin(OffScheduleAdmin):
    form = OffScheduleCommForm



import re

from django import forms
from django.utils.html import format_html
from edc_constants.constants import UUID_PATTERN
from edc_form_validators import FormValidatorMixin
from edc_screening.modelform_mixins import AlreadyConsentedFormMixin
from edc_sites.modelform_mixins import SiteModelFormMixin
from intecomm_form_validators import PatientLogFormValidator

from ..models import PatientGroup, PatientLog


class PatientLogForm(
    AlreadyConsentedFormMixin, SiteModelFormMixin, FormValidatorMixin, forms.ModelForm
):
    form_validator_cls = PatientLogFormValidator

    def clean(self) -> dict:
        if self.instance.id:
            qs = PatientGroup.objects.filter(patients__in=[self.instance])
            if qs.count() > 0:
                raise forms.ValidationError(
                    format_html(
                        "Changes not allowed. Patient is already in group <A href="
                        f"{qs[0].get_absolute_url()}>{qs[0].name}</A>."
                    )
                )
        return super().clean()

    def get_initial_for_field(self, field, field_name):
        """Set existing data value to None if UUID.

        Required for forms that were saved when the
        name fields were hidden by mistake.
        """
        value = super().get_initial_for_field(field, field_name)
        if value and field_name in ["legal_name", "familiar_name"]:
            if field_name in [fld.name for fld in self.visible_fields()]:
                if re.match(UUID_PATTERN, value):
                    value = None
        return value

    class Meta:
        model = PatientLog
        fields = "__all__"



# Generated by Django 4.1.2 on 2022-11-22 20:43

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0009_historicalsubjectscreening_familiar_name_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalpatientlog",
            name="age_in_years",
            field=models.IntegerField(default=0, verbose_name="Age"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="patientlog",
            name="age_in_years",
            field=models.IntegerField(default=0, verbose_name="Age"),
            preserve_default=False,
        ),
    ]



# Generated by Django 4.2.3 on 2023-08-12 23:56

from django.db import migrations, models
import edc_model.validators.date
import edc_protocol.validators


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0102_alter_healtheconomicshouseholdhead_hoh_education_old_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_status",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="Household head’s employment status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_employment_status",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your employment status?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_status",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="Household head’s employment status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_employment_status",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your employment status?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisitmissed",
            name="contact_last_date",
            field=models.DateField(
                blank=True,
                null=True,
                validators=[
                    edc_model.validators.date.date_not_future,
                    edc_protocol.validators.date_not_before_study_start,
                ],
                verbose_name="Date of last telephone contact/attempt",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisitmissed",
            name="contact_last_date",
            field=models.DateField(
                blank=True,
                null=True,
                validators=[
                    edc_model.validators.date.date_not_future,
                    edc_protocol.validators.date_not_before_study_start,
                ],
                verbose_name="Date of last telephone contact/attempt",
            ),
        ),
    ]



# Generated by Django 4.2.1 on 2023-06-30 22:17

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_lists", "0006_consentrefusalreasons_screeningrefusalreasons_and_more"),
        (
            "intecomm_screening",
            "0036_alter_historicalpatientlog_screening_identifier_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="consentrefusal",
            name="screening_identifier",
            field=models.CharField(max_length=36, unique=True),
        ),
        migrations.AlterField(
            model_name="consentrefusal",
            name="subject_screening",
            field=models.OneToOneField(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="intecomm_screening.subjectscreening",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsentrefusal",
            name="screening_identifier",
            field=models.CharField(db_index=True, max_length=36),
        ),
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="willing_to_screen",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                max_length=15,
                verbose_name="Has the patient agreed to be screened for the INTECOMM study",
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="screening_refusal_reason",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="intecomm_lists.screeningrefusalreasons",
                verbose_name="Reason subject unwilling to screen",
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="willing_to_screen",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                max_length=15,
                verbose_name="Has the patient agreed to be screened for the INTECOMM study",
            ),
        ),
    ]



from __future__ import annotations

from django.db.models.signals import post_delete, post_save, pre_delete
from django.dispatch import receiver
from edc_utils import get_uuid

from . import PatientLog
from .patient_call import PatientCall
from .subject_screening import SubjectScreening


class SubjectScreeningDeleteError(Exception):
    pass


class PatientLogDeleteError(Exception):
    pass


@receiver(
    post_save,
    weak=False,
    sender=SubjectScreening,
    dispatch_uid="update_subjectscreening_on_post_save",
)
def update_subjectscreening_on_post_save(sender, instance, raw, **kwargs):
    if not raw:
        instance.patient_log.screening_identifier = instance.screening_identifier
        instance.patient_log.screening_datetime = instance.report_datetime
        instance.patient_log.save_base(
            update_fields=["screening_identifier", "screening_datetime"]
        )


@receiver(
    pre_delete,
    weak=False,
    sender=PatientLog,
    dispatch_uid="patientlog_on_pre_delete",
)
def patientlog_on_pre_delete(sender, instance, **kwargs):
    if instance.screening_identifier:
        raise PatientLogDeleteError(
            f"Not allowed. Subject has been screened. Got {instance.screening_identifier}."
        )


@receiver(
    pre_delete,
    weak=False,
    sender=SubjectScreening,
    dispatch_uid="subjectscreening_on_pre_delete",
)
def subjectscreening_on_pre_delete(sender, instance, **kwargs):
    if instance.consented:
        raise SubjectScreeningDeleteError(
            f"Not allowed. Subject is consented. Got {instance.screening_identifier}."
        )


@receiver(
    post_delete,
    weak=False,
    sender=SubjectScreening,
    dispatch_uid="subjectscreening_on_post_delete",
)
def subjectscreening_on_post_delete(sender, instance, **kwargs):
    instance.patient_log.screening_identifier = get_uuid()
    instance.patient_log.screening_datetime = None
    instance.patient_log.save_base(
        update_fields=["screening_identifier", "screening_datetime"]
    )


@receiver(
    post_save,
    weak=False,
    sender=PatientCall,
    dispatch_uid="patient_call_on_post_save",
)
def patient_call_on_post_save(sender, instance, raw, created, **kwargs):
    if not raw:
        qs = sender.objects.filter(patient_log=instance.patient_log)
        instance.patient_log.call_attempts = qs.count()
        instance.patient_log.save(update_fields=["call_attempts"])


@receiver(
    post_delete,
    weak=False,
    sender=PatientCall,
    dispatch_uid="patient_call_on_post_delete",
)
def patient_call_on_post_delete(sender, instance, using, **kwargs):
    instance.patient_log.call_attempts = (
        0
        if instance.patient_log.call_attempts == 0
        else instance.patient_log.call_attempts - 1
    )
    instance.patient_log.save(update_fields=["call_attempts"])



from django.contrib import admin

from ..admin_site import intecomm_prn_admin
from ..models import OnScheduleComm
from .modeladmin_mixins import OnScheduleModelAdmin


@admin.register(OnScheduleComm, site=intecomm_prn_admin)
class OnScheduleCommAdmin(OnScheduleModelAdmin):
    pass



# Generated by Django 4.2.3 on 2023-08-03 00:05
from django.db import migrations
from edc_list_data import PreloadData
from tqdm import tqdm

from intecomm_lists.list_data import list_data


def update_ethnicity_on_he(apps, schema_editor):
    list_model_cls = apps.get_model("edc_he.ethnicities")
    list_model_cls.objects.filter(name="mkuria-mshashi").update(name="mkuria_mshashi")
    list_model_cls.objects.filter(name="mkara-mkerewe").update(name="mkara_mkerewe")
    ld = {"edc_he.ethnicities": list_data.get("edc_he.ethnicities")}
    PreloadData(list_data=ld, apps=apps)
    household_head_cls = apps.get_model("intecomm_subject.healtheconomicshouseholdhead")
    total = household_head_cls.objects.all().count()
    for obj in tqdm(household_head_cls.objects.all(), total=total):
        list_model_obj = list_model_cls.objects.get(name=obj.hoh_ethnicity_old)
        obj.hoh_ethnicity_id = list_model_obj.id
        obj.save_base(update_fields="hoh_ethnicity")


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0071_healtheconomicshouseholdhead_hoh_ethnicity_and_more"),
    ]

    operations = [migrations.RunPython(update_ethnicity_on_he)]



# Generated by Django 4.1.7 on 2023-05-11 16:22

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_group", "0011_alter_historicalpatientfollowupcall_respondent_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalpatientgroup",
            name="status",
            field=models.CharField(
                choices=[
                    ("New", "New"),
                    ("recruiting", "Recruiting"),
                    ("COMPLETE", "Complete/Ready"),
                    ("in_followup", "In followup"),
                    ("dissolved", "Dissolved"),
                ],
                default="recruiting",
                help_text="Will reverted to 'Recruiting' if changes are made to the list of selected patients",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="patientgroup",
            name="status",
            field=models.CharField(
                choices=[
                    ("New", "New"),
                    ("recruiting", "Recruiting"),
                    ("COMPLETE", "Complete/Ready"),
                    ("in_followup", "In followup"),
                    ("dissolved", "Dissolved"),
                ],
                default="recruiting",
                help_text="Will reverted to 'Recruiting' if changes are made to the list of selected patients",
                max_length=25,
            ),
        ),
    ]



from django.apps import AppConfig as DjangoAppConfig


class AppConfig(DjangoAppConfig):
    name = "intecomm_dashboard"
    verbose_name = "INTECOMM: Dashboard"
    admin_site_name = "meta_test_admin"
    include_in_administration_section = False



from django.contrib.auth.models import User
from django.urls import reverse
from django_webtest import WebTest


class TestNextAppointment(WebTest):
    def setUp(self) -> None:
        super().setUp()
        self.user = User.objects.create_superuser("user_login", "u@example.com", "pass")
        self.user.is_active = True
        self.user.is_staff = True
        self.user.save()
        self.user.refresh_from_db()

    def login(self):
        form = self.app.get(reverse("admin:index")).maybe_follow().form
        form["username"] = self.user.username
        form["password"] = "pass"  # nosec B105
        return form.submit()

    def test_admin_ok(self):
        self.login()



# Generated by Django 4.2.3 on 2023-07-26 13:31

import edc_facility.model_mixins
import edc_sites.model_mixins
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_facility", "0002_auto_20230726_2030"),
        (
            "intecomm_screening",
            "0048_rename_historicalhealthfacility_historicaloldhealthfacility_and_more",
        ),
    ]

    operations = [
        migrations.RenameModel(
            old_name="HistoricalHealthFacility",
            new_name="HistoricalOldHealthFacility",
        ),
        migrations.RenameModel(
            old_name="HealthFacility",
            new_name="OldHealthFacility",
        ),
    ]



from __future__ import annotations

from edc_constants.constants import COMPLETE

from ..exceptions import PatientGroupError
from .patient_group import PatientGroup


def add_to_group(instance, commit: bool | None = None):
    commit = True if commit is None else commit
    to_group = instance.patient_group
    if through_qs := instance.patientgroup_set.through.objects.filter(
        patientlog_id=instance.id
    ):
        if to_group and to_group.id != through_qs.first().patientgroup_id:
            if not instance.patient_group.patients.filter(name=instance.name).exists():
                if commit:
                    instance.patient_group.patients.add(instance)
            else:
                raise PatientGroupError(
                    "Cannot add patient to group. Already a member of this group"
                )
    else:
        if commit:
            instance.patient_group.patients.add(instance)


def remove_from_group(instance, commit: bool | None = None):
    """Updates patient_group.patients(m2m) using
    patient_log.patient_group(fk).
    """
    commit = True if commit is None else commit
    to_group = instance.patient_group
    if through_qs := instance.patientgroup_set.through.objects.filter(
        patientlog_id=instance.id
    ):
        ids = [obj.patientgroup_id for obj in through_qs]
        if from_patient_group_qs := PatientGroup.objects.filter(id__in=ids):
            if to_group and from_patient_group_qs.first().id == to_group.id:
                pass
            else:
                if COMPLETE in [obj.status for obj in from_patient_group_qs]:
                    raise PatientGroupError(
                        "Cannot remove patient from group. See patient group "
                        f"{from_patient_group_qs.first()}."
                    )
                for obj in from_patient_group_qs:
                    if commit:
                        obj.patients.remove(instance)



# Generated by Django 4.2.3 on 2023-08-15 01:45

from django.db import migrations, models
import django.db.models.deletion
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("edc_adverse_event", "0012_aeactionclassification_extra_value_and_more"),
        ("intecomm_ae", "0006_hospitalization_historicalhospitalization"),
    ]

    operations = [
        migrations.AddField(
            model_name="aeinitial",
            name="ae_classification_as_text",
            field=models.CharField(
                help_text="Keep this simple. Provide details below",
                max_length=150,
                null=True,
                verbose_name="Adverse Event (AE) Classification",
            ),
        ),
        migrations.AddField(
            model_name="historicalaeinitial",
            name="ae_classification_as_text",
            field=models.CharField(
                help_text="Keep this simple. Provide details below",
                max_length=150,
                null=True,
                verbose_name="Adverse Event (AE) Classification",
            ),
        ),
        migrations.AlterField(
            model_name="aeinitial",
            name="ae_classification",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_adverse_event.aeclassification",
                verbose_name="Adverse Event (AE) Classification",
            ),
        ),
        migrations.AlterField(
            model_name="aeinitial",
            name="ae_classification_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                editable=False,
                max_length=250,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="historicalaeinitial",
            name="ae_classification",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_adverse_event.aeclassification",
                verbose_name="Adverse Event (AE) Classification",
            ),
        ),
        migrations.AlterField(
            model_name="historicalaeinitial",
            name="ae_classification_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                editable=False,
                max_length=250,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
    ]



from django.contrib import admin
from edc_lab_results.fieldsets import BloodResultFieldset

from ..admin_site import intecomm_subject_admin
from ..forms import ViralLoadResultForm
from ..models import ViralLoadResult
from .modeladmin_mixins import CrfModelAdmin


@admin.register(ViralLoadResult, site=intecomm_subject_admin)
class ViralLoadResultAdmin(CrfModelAdmin):
    form = ViralLoadResultForm
    autocomplete_fields = ["requisition"]
    fieldsets = BloodResultFieldset(
        ViralLoadResult.lab_panel,
        model_cls=ViralLoadResult,
        exclude_units=True,
        exclude_reportable=True,
    ).fieldsets



from django.contrib import admin
from edc_constants.constants import INTEGRATED
from edc_facility.utils import get_health_facility_model_cls
from edc_next_appointment.modeladmin_mixins import NextAppointmentModelAdminMixin

from ..admin_site import intecomm_subject_admin
from ..forms import NextAppointmentForm
from ..models import NextAppointment
from .modeladmin_mixins import CrfModelAdmin


@admin.register(NextAppointment, site=intecomm_subject_admin)
class NextAppointmentAdmin(NextAppointmentModelAdminMixin, CrfModelAdmin):
    form = NextAppointmentForm

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "health_facility":
            kwargs["queryset"] = get_health_facility_model_cls().on_site.filter(
                health_facility_type__name__in=[INTEGRATED]
            )
        return super().formfield_for_foreignkey(db_field, request, **kwargs)
